#-*- coding:utf-8 -*-

"""
Provides functions to handle source code highlighting.
"""

from pygments import highlight
from pygments.lexers import get_lexer_by_name, PythonLexer
from pygments.formatters import HtmlFormatter
from pygments.styles import get_style_by_name

from pydozeoff.conf import settings


def get_style_defs(name="default"):
    """Returns the CSS code used to highlight source code. If the given style
    name isn't found, the default style will be used.
    """
    try:
        return HtmlFormatter(style=name).get_style_defs()
    except:
        return HtmlFormatter().get_style_defs()


def code(language, source_code):
    """Formats the given source code snippet as HTML.
    """
    formatter = HtmlFormatter(**settings["SYNTAX_HIGHLIGHT_OPTIONS"])
    return highlight(source_code, _get_lexer(language, source_code), formatter)


def _get_lexer(language, source_code):
    """Returns the appropriate lexer to parse the given source code snippet.
    """
    try:
        lexer = get_lexer_by_name(language)
    except:
        try:
            lexer = guess_lexer(source_code)
        except:
            lexer = PythonLexer()
    return lexer



#-*- coding:utf-8 -*-

"""
Provides a web interface to generate and serve a presentation.
"""

import os

from bottle import route, send_file, run, debug

from pydozeoff.template import template_engine
from pydozeoff.conf import settings


MEDIA_NAMESPACE = "/media"
THEME_NAMESPACE = "/theme"


def start(port=8080, debug_mode=False):
    """Starts the server.
    """
    debug(debug_mode)
    run(host="0.0.0.0", port=port, reloader=debug_mode)


@route("/")
def index():
    """Assembles the presentation based on the slideshow settings.
    """
    slides = []
    for slide in settings["SLIDES"]:
        context = {}
        slide_filename = _get_slide_template(slide.path)
        context["content"] = template_engine.render(slide_filename, slide.options)
        context.update(slide.options)
        slides.append(context)
    return template_engine.render(_get_slideshow_template(), locals())


@route("%s/(?P<filename>.+)" % MEDIA_NAMESPACE)
def serve_media(filename):
    """Serves static files from the media directory.
    """
    send_file(filename, root="%(ROOT_DIR)s/%(MEDIA_DIR)s" % settings)


@route("%s/(?P<filename>.+)" % THEME_NAMESPACE)
def serve_template_media(filename):
    """Serves static files from the theme directory.
    """
    send_file(filename, root="%(ROOT_DIR)s/%(THEMES_DIR)s/%(THEME)s" % settings)


def _get_slideshow_template():
    """Returns the slideshow template file.
    """
    return "%(THEMES_DIR)s/template.html" % settings

def _get_slide_template(slide_partial_path):
    """Returns the path for the given relative path.
    """
    path = "%s/%s" % (settings["SLIDES_DIR"], slide_partial_path)
    return os.path.normpath(path).replace(os.sep, "/")



#-*- coding:utf-8 -*-

"""
Provides a template renderer implementation.
"""

from jinja2 import Environment, Template, FileSystemLoader

from pydozeoff.conf import settings


class Jinja2TemplateEngine(object):
    """Jinja2-based template engine.
    """
    def __init__(self):
        self.initialized = False

    def initialize(self):
        """Initializes the template engine.
        """
        if not self.initialized:
            self.initialized = True
            self.env =  Environment(
                loader      = FileSystemLoader(settings["ROOT_DIR"], encoding=settings["ENCODING"]),
                extensions  = settings["TEMPLATE_ENGINE_EXTENSIONS"],
                auto_reload = settings["DEBUG_MODE"],
            )
            self.env.globals = settings

            self.env.filters.update(settings["TEMPLATE_ENGINE_FILTERS"])
            self.env.tests.update(settings["TEMPLATE_ENGINE_TESTS"])

    def render(self, template_file, context={}):
        """Renders a template.
        """
        template = self.get_template(template_file)
        return template.render(context)

    def get_template(self, template_file):
        """Creates a template object from a file.
        """
        self.initialize()
        return self.env.get_template(template_file)


template_engine = Jinja2TemplateEngine()



#-*- coding:utf-8 -*-

"""
Slideshow settings file.
"""

# Slide definition shortcuts. See "SLIDES" below for more information
from pydozeoff.conf.slide import *


# If you want to see all settings available and their default values, please
# import the "global_settings" module, like this:
#
#     >>> from pydozeoff.conf import global_settings
#     >>> dir(global_settings)
#     ...list of settings...
#     >>> global_settings.DEFAULT_VIEW
#     'slideshow'


# All settings defined in this file are exposed to slide templates

# Presentation info
TITLE    = u"Presentation title"
SPEAKER  = u"Author name"
COMPANY  = u"Company"
LOCATION = u"Location"
DATE     = u"yyyy-mm-dd"

# Feel free to add your own settings
MY_SETTING = u"Some text here!"

# Copy a S5 theme to the "themes" directory and change this setting accordingly
# Visit http://meyerweb.com/eric/tools/s5/ for more information
THEME = "theme_name"

# Presentation slides
SLIDES = slides(

    # Each entry points to a file under the "slides" directory
    # simple("first.html"),

    # There are shortcut functions to the most common slide types. All they do
    # is add style classes to the slide <div>, so you might need to tweak the
    # CSS files of your theme to handle them properly

    # bullets("file.html"), # For bullet point slides
    # simple ("file.html"), # For short text sentences
    # image  ("file.html"), # For images
    # code   ("file.html"), # For source code snippets

    # You can use keyword args to complement the slide template context
    # bullets("second.html", custom_title="Configurable title"),

    # You can also separate your slides into subdirectories, to keep them
    # organized. Feel free to nest as many sections as you want
    # section("subsection",
    #     code ("first.html"),  # Points to: slides/subsection/first.html
    #     image("second.html"), # Points to: slides/subsection/second.html
    # ),
)



#-*- coding:utf-8 -*-

"""
Most common slide types.
"""

__all__ = ["slides", "section", "simple", "bullets", "code", "image",]

def slides(*elements):
    """Defines the sequence of slides for the presentation.
    """
    return _collect_slides(elements, [])

def _collect_slides(elements, current):
    """Recursively collects all slides.
    """
    for element in elements:
        if isinstance(element, Slide):
            current.append(element)
        elif isinstance(element, Section):
            _collect_slides(element.elements, current)
    return current


def section(name, *elements):
    """Defines a parent section, which is just a parent directory.
    """
    return Section(name, *elements)

def simple(filename, **kwargs):
    """Defines a slide composed of a couple lines of text.
    """
    opt = {"classes": "simple"}; opt.update(kwargs)
    return Slide(filename, **opt)

def bullets(filename, **kwargs):
    """Configures a slide composed of bullet points.
    """
    opt = {"classes": "bullets"}; opt.update(kwargs)
    return Slide(filename, **kwargs)

def code(filename, **kwargs):
    """Configures a slide composed of source code snippets.
    """
    opt = {"classes": "code"}; opt.update(kwargs)
    return Slide(filename, **opt)

def image(filename, **kwargs):
    """Configures a slide that displays a image.
    """
    opt = {"classes": "image"}; opt.update(kwargs)
    return Slide(filename, **opt)


class Element(object):
    def __init__(self, name):
        self._name = name
        self._parent = None

    def set_parent(self, parent):
        self._parent = parent

    def get_parent(self):
        return self._parent

    def get_path(self):
        if not self._parent:
            return self._name
        return "%s/%s" % (self.parent.path, self._name)

    parent = property(get_parent, set_parent)
    path   = property(get_path)


class Section(Element):
    def __init__(self, name, *elements):
        super(Section, self).__init__(name)
        for element in elements:
            element.set_parent(self)
        self._elements = elements

    def get_elements(self):
        return self._elements

    elements = property(get_elements)

class Slide(Element):
    def __init__(self, name, **kwargs):
        super(Slide, self).__init__(name)
        self._options = kwargs

    def get_options(self):
        return self._options

    options = property(get_options)



#-*- coding:utf-8 -*-

"""
Provides useful template engine extensions.
"""

from jinja2 import nodes
from jinja2.ext import Extension

from pydozeoff.template import highlight
from pydozeoff.conf import settings


class CodeHighlightExtension(Extension):
    """Extension that highlights source code.
    """
    tags = set(["code"])

    def __init__(self, environment):
        super(CodeHighlightExtension, self).__init__(environment)

    def parse(self, parser):
        lineno = parser.stream.next().lineno

        # Optional argument to specify the programming language
        args = []
        if not parser.stream.current.test("block_end"):
            args.append(parser.parse_expression())
        else:
            args.append(nodes.Const(None))

        body = parser.parse_statements(['name:endcode'], drop_needle=True)
        return nodes.CallBlock(self.call_method('_highlight_support', args), \
            [], [], body).set_lineno(lineno)

    def _highlight_support(self, language, caller):
        code = caller()
        return highlight.code(language, code)


class CodeHighlightStyleExtension(Extension):
    """Extension that renders the CSS used to style highlighted source code.
    """
    tags = set(["code_highlight_css"])

    def __init__(self, environment):
        super(CodeHighlightStyleExtension, self).__init__(environment)

    def parse(self, parser):
        lineno = parser.stream.next().lineno
        return nodes.CallBlock(self.call_method('_highlight_style_support', []), \
            [], [], []).set_lineno(lineno)

    def _highlight_style_support(self, caller):
        return '<style type="text/css">%s</style>' % \
            highlight.get_style_defs(settings["SYNTAX_HIGHLIGHT_OPTIONS"]["style"])


# Aliases for extensions, filters and tests
code          = CodeHighlightExtension
code_style    = CodeHighlightStyleExtension



#-*- coding:utf-8 -*-

"""
pydozeoff root module.
"""

import os
import shutil
import sys


__version__   = "0.1.2"
__author__    = "Daniel Fernandes Martins"
__license__   = "BSD"
__copyright__ = "Copyright 2010, Destaquenet Technology Solutions"


def main():
    """Entry point for the command line tool.
    """
    parser, options = _get_cmdline_options()
    if options.create:
        _create_presentation(parser, options)
    elif options.build:
        _build_presentation(parser, options)
    else:
        _serve_presentation(parser, options)


def _create_presentation(parser, options):
    """Create a new presentation.
    """
    module_dir = os.path.dirname(__file__)
    shutil.copytree("%s/conf/slideshow_template" % module_dir, options.create)
    print >> sys.stderr, "Presentation '%s' created successfuly" % options.create


def _build_presentation(parser, options):
    """Builds an offline version of a presentation.
    """
    _check_for_settings(parser, options)
    _prepare_env(options)

    from pydozeoff import webapp
    settings = webapp.settings

    # Sanity checks
    if os.path.exists(options.build):
        if os.path.isfile(options.build):
            print >> sys.stderr, "Output directory '%s' must not be a file" % options.build
            sys.exit(1)
        elif os.path.isdir(options.build):
            print >> sys.stderr, "Output directory '%s' must not already exist" % options.build
            sys.exit(1)

    # Copy theme
    theme_from = "%s/%s/%s" % (options.slideshow_path, settings["THEMES_DIR"], settings["THEME"])
    theme_to   = "%s%s" % (options.build, webapp.THEME_NAMESPACE)
    shutil.copytree(theme_from, theme_to)

    # Copy media
    media_from = "%s/%s" % (options.slideshow_path, settings["MEDIA_DIR"])
    media_to   = "%s%s" % (options.build, webapp.MEDIA_NAMESPACE)
    shutil.copytree(media_from, media_to)

    # Presentation HTML
    out = open("%s/index.html" % options.build, mode="w+")
    out.write(webapp.index().encode(settings["ENCODING"]))
    out.close()

    print >> sys.stderr, "Offline version generated at '%s'" % options.build


def _serve_presentation(parser, options):
    """Bootstraps the app.
    """
    _check_for_settings(parser, options)
    _prepare_env(options)

    from pydozeoff import webapp
    webapp.start(port=options.server_port, debug_mode=options.debug)


def _check_for_settings(parser, options):
    """Checks whether the slideshow settings module exists.
    """
    module_path = "%s/slideshow.py" % options.slideshow_path
    if not os.path.exists(module_path):
        print >> sys.stderr, "File not found: %s" % module_path
        parser.print_help()
        sys.exit(1)


def _prepare_env(options):
    """Prepares the environment.
    """
    sys.path.append(options.slideshow_path)
    os.environ["PYDOZEOFF_ROOT_DIR"]   = options.slideshow_path
    os.environ["PYDOZEOFF_DEBUG_MODE"] = ["","1"][options.debug]


def _get_cmdline_options():
    """Parses command line options.
    """
    import optparse
    parser = optparse.OptionParser()

    parser.add_option("--slideshow-path", "-s", default=".",
        help="path to the presentation root directory (the one with a "
             "slideshow.py file)")

    # Build presentation
    build = optparse.OptionGroup(parser, "Starting/building presentations")
    parser.add_option_group(build)

    build.add_option("--create", "-c", default="",
        help="start a new presentation")
    build.add_option("--build", "-b", default="",
        help="create offline version of a presentation at the given directory")

    # Serve presentation
    serve = optparse.OptionGroup(parser, "Serving presentations")
    parser.add_option_group(serve)

    serve.add_option("--server-port", "-p", default="8080",
        help="port used by the webserver")
    serve.add_option("--debug", "-d", action="store_true", default=False,
        help="start server in debug mode")

    return (parser, parser.parse_args()[0])


    return (parser, parser.parse_args()[0])



#-*- coding:utf-8 -*-

# Slideshow settings file

# Presentation info
TITLE    = u""
SPEAKER  = u""
COMPANY  = u""
LOCATION = u""
DATE     = u""

# Directory structure
ROOT_DIR      = "."
SLIDES_DIR    = "slides"
MEDIA_DIR     = "media"
THEMES_DIR    = "themes"

# Engine settings
THEME        = "default"
CONTROL_VIS  = "hidden"
DEFAULT_VIEW = "slideshow"

ENCODING = "utf-8"

# Syntax highlight settings
SYNTAX_HIGHLIGHT_OPTIONS = {
    "style": "default",
}

# Template engine extensions, filters and tests
TEMPLATE_ENGINE_EXTENSIONS = [
    "pydozeoff.template.ext.code",
    "pydozeoff.template.ext.code_style",
]

TEMPLATE_ENGINE_FILTERS = {}
TEMPLATE_ENGINE_TESTS = {}

# Slides
SLIDES = ()



#-*- coding:utf-8 -*-

"""
Provides classes to handle slideshow settings.
"""

import os

import pydozeoff
from pydozeoff.conf import global_settings

# Slideshow settings
import slideshow as _slideshow_settings


class LazySettings(dict):
    """Implements the slideshow settings dictionary.
    """
    def __init__(self):
        self.initialized = False

    def initialize(self):
        """Initializes this object if it's not yet initialized.
        """
        if not self.initialized:
            self.initialized = True
            self._copy_members(global_settings)
            self._copy_members(_slideshow_settings)
            self._copy_env()


    def _copy_members(self, module):
        """Copies public members from a module to this settings object.
        """
        for name in dir(module):
            if name.startswith("_"):
                continue
            self[name] = getattr(module, name)

    def _copy_env(self):
        """Copies important environment variables to this settings object.
        """
        self["VERSION"] = pydozeoff.__version__
        self["ROOT_DIR"]   = os.environ.get("PYDOZEOFF_ROOT_DIR", ".")
        self["DEBUG_MODE"] = bool(os.environ.get("PYDOZEOFF_DEBUG_MODE", ""))

    def __getitem__(self, item):
        """Returns the corresponding setting, and initialize this object if
        necessary.
        """
        self.initialize()
        return super(LazySettings, self).__getitem__(item)


settings = LazySettings()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, find_packages

setup(
    name = "pydozeoff",
    version = "0.1.2",
    author = "Daniel Fernandes Martins",
    author_email = 'daniel@destaquenet.com',
    url = "http://github.com/danielfm/pydozeoff",
    description = "Web-based presentation engine for programmers",
    license = "BSD",
    keywords = ["web", "presentation", "slideshow", "talk", "engine", "python", "s5"],
    classifiers = [
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.5",
        "Topic :: Communications",
        "Topic :: Multimedia :: Graphics :: Presentation",
    ],
    platforms = [
        "Any",
    ],

    # Dependencies
    install_requires = ["bottle>=0.6", "Jinja2>=2.3", "Pygments>=1.3"],

    # Executable scripts
    entry_points = {
        'console_scripts': [
            "pydozeoff = pydozeoff:main"
        ],
    },

    # Source code structure
    packages = find_packages("src"),
    package_dir = {"": "src"},
    include_package_data = True,
    zip_safe = False,

    long_description = 
"""
As a Linux user, I don't have access to so called "professional" presentation
softwares such as Apple Keynote or MS PowerPoint. OpenOffice Impress used to be
my first choice, but I gave up using it since it's a memory/processor hog for
medium-sized presentations.

After a little research, I came across a wonderful tool called `S5`_. It was a
perfect fit, except for the one-big-fat-presentation-html thing and the lack of
syntax highlighting.

This is what I'm trying to solve with this project.

.. _Python: http://python.org/
.. _S5: http://meyerweb.com/eric/tools/s5/
""")
